<?php
/*
 Plugin Name: Deferred RSS
 Plugin URI: http://blog.sjinks.pro/wordpress/plugins/424-deferred-feed-publishing/
 Description: Defers RSS publishing (and allows to add your copyright notice to the feed)
 Author: Vladimir Kolesnikov
 Version: 1.1
 Author URI: http://blog.sjinks.pro/
 */

	class DeferredRss
	{
		const option_name = 'sjdr_options';

		protected $options;

		public static function instance()
		{
			static $self = false;

			if (!$self) {
				$self = new DeferredRss();
			}

			return $self;
		}

		public function __construct()
		{
			add_action('init', array($this, 'init'));
		}

		public function init()
		{
			add_filter('posts_where',          array($this, 'posts_where'));
			add_filter('get_lastpostmodified', array($this, 'get_lastpostmodified'), 10, 2);

			add_filter('the_content',     array($this, 'replace'), 100);
			add_filter('the_content_rss', array($this, 'replace'), 100);
			add_filter('get_the_excerpt', array($this, 'replace'), 100);

			load_plugin_textdomain('sjdrss', PLUGINDIR . '/deferred-rss/lang');

			if (is_admin()) {
				add_action('admin_init', array($this, 'admin_init'));
				add_action('admin_menu', array($this, 'admin_menu'));
			}
		}

		public function admin_init()
		{
			add_action('load-deferred-rss/options-dr.php',  array($this, 'load_options_page'));
			add_action('admin_post_sjdrss_save_settings', array($this, 'sjdrss_save_settings'));
		}

		public function get_lastpostmodified($lastpostmodified, $timezone)
		{
			if (is_feed() && 'gmt' == strtolower($timezone)) {
				global $wpdb;
				$gmt_offset = intval(get_option('gmt_offset')) * 3600;
				$now = time() + $gmt_offset;

				$options = $this->loadOptions();
				if ($options['time'] > 0) {
					$lmt = strtotime($lastpostmodified);
					switch (strtolower($options['UNITS'])) {
						case 'DAY':  $amd = 24*3600; break;
						case 'HOUR': $amd = 3600; break;
						default:     $amd = 60; break;
					}

					$lmt += intval($options['time']) * $amd;

					// If post modification time plus delay is in the past, we return the actual date - no idea why but why not?
					if ($lmt < $now) {
						return $lastpostmodified;
					}

					$lmt = $now - intval($options['time']) * $amd;
					$lmt = date('Y-m-d H:i:s', $lmt);

					$lastpostmodified = $wpdb->get_var("SELECT post_modified_gmt FROM $wpdb->posts WHERE post_status = 'publish' AND post_modified_gmt <= {$lmt} ORDER BY post_modified_gmt DESC LIMIT 1");
				}
			}

			return $lastpostmodified;
		}

		public function posts_where($q)
		{
			if (is_feed()) {
				global $wpdb;

				$options = $this->loadOptions();
				$time    = (int)$options['time'];
				$units   = $wpdb->escape($options['units']);

				$q  .= " AND `{$wpdb->posts}`.`post_date` < DATE_SUB(NOW(), INTERVAL {$time} {$units}) ";
			}

			return $q;
		}

		public function replace($s)
		{
			if (is_feed()) {
				$options = $this->loadOptions();
				$to_add  = sprintf($options['copyrights'], date('Y'), get_option('home'), get_option('blogname'), esc_attr(get_permalink()));

				$s = preg_replace('/\\[is_feed\\](.*?)\\[\\/is_feed\\]/s', '$1', $s);
				$s = preg_replace('/\\[is_not_feed\\].*?\\[\\/is_not_feed\\]/s', '', $s);

				$s = preg_replace('/<!\\-\\-is_feed>(.*?)<\\/is_feed\\-\\->/s', '$1', $s);
				$s = preg_replace('/<!\\-\\-is_not_feed\\-\\->.*?<!\\-\\-\\/is_not_feed\\-\\->/s', '', $s);

				if ($options['rus_replace']) {
					$replace = array(
						'а' => 'a', 'е' => 'e', 'о' => 'o', 'р' => 'p', 'с' => 'c',
						'у' => 'y', 'х' => 'x', 'А' => 'A', 'В' => 'B', 'Е' => 'E',
						'К' => 'K', 'М' => 'M', 'Н' => 'H', 'О' => 'O', 'Р' => 'P',
						'С' => 'C', 'Т' => 'T', 'Х' => 'X',
					);

					$s = str_replace(array_keys($replace), array_values($replace), $s);
				}

				$s .= $to_add;
			}
			else {
				$s = preg_replace('/\\[is_feed\\].*?\\[\\/is_feed\\]/s', '', $s);
				$s = preg_replace('/\\[is_not_feed\\](.*?)\\[\\/is_not_feed\\]/s', '$1', $s);
				$s = preg_replace('/<!\\-\\-is_feed>.*?<\\/is_feed\\-\\->/s', '', $s);
				$s = preg_replace('/<!\\-\\-is_not_feed\\-\\->(.*?)<!\\-\\-\\/is_not_feed\\-\\->/s', '$1', $s);
			}

			return $s;
		}


		public function admin_menu()
		{
			add_options_page(__('Deferred RSS Options', 'sjdrss'), 'Deferred RSS', 'manage_options', 'deferred-rss/options-dr.php');
		}

		public function loadOptions()
		{
			static $done = false;

			if (!$done) {
				$done     = true;
				$defaults = array(
					'time'        => 30,
					'units'       => 'MINUTE',
					'copyrights'  => __('<p>&copy; %1$d <a href="%2$s">%3$s</a>. All rights reserved.</p>', 'sjdrss'),
					'rus_replace' => 0,
				);

				$options = get_option(DeferredRss::option_name);
				if (!is_array($options)) {
					$options = $defaults;
					add_option(DeferredRss::option_name, $defaults, '', false);
				}

				$update = false;
				foreach ($defaults as $k => $v) {
					if (!isset($options[$k])) {
						$options[$k] = $v;
						$update = true;
					}
				}

				foreach ($options as $k => $v) {
					if (!isset($defaults[$k])) {
						unset($options[$k]);
						$update = true;
					}
				}

				if ($update) {
					update_option(DeferredRss::option_name, $options);
				}

				$this->options = $options;
			}

			return $this->options;
		}

		public function writeOptions()
		{
			update_option(DeferredRss::option_name, $this->options);
		}

		public function load_options_page()
		{
			if (!current_user_can('manage_options')) {
				wp_die(__("You do not have enough privileges to do this.", 'sjdrss'));
			}

			global $options;
			$options = $this->loadOptions();
		}

		public function sjdrss_save_settings()
		{
			$this->loadOptions();

			if (!empty($_POST['options'])) {
				check_admin_referer('save-sjdrss_settings');

				foreach ($this->options as $key => $value) {
					$this->options[$key] = isset($_POST['options'][$key]) ? stripslashes($_POST['options'][$key]) : $value;
				}

				$this->writeOptions();
			}

			wp_redirect(admin_url('options-general.php?page=deferred-rss/options-dr.php&updated=1'));
			die();
		}
	}

	DeferredRss::instance();
?>