<?php
/*
Plugin Name: Terms Descriptions
Plugin URI: http://localhost
Description: Этот плагин ищет заданные термины в постах и превращает их в ссылки на соответствующие страницы.
Version: 3.0
Author: Vladimir Statsenko
Author URI: http://www.simplecoding.org
License: GPL2
*/

/*
 После активации плагин создает в админке ( в разделе Tools ) страницу с формой
 создания новых терминов и управления существующими.
 
 Терминами могут быть любые выражения, содержащие одно или несколько слов.
 Каждому термину назначается ссылка на страницу с пояснениями. Эти страницы
 должны быть созданы заранее.
 
 При формировании постов, плагин ищет в них указанные термины и преобразовывает
 их в ссылки.
 
 Термины игнорируются в следующих случаях.
 1 ) Если они уже находятся внутри ссылки.
 2 ) Если они находятся внутри тега ( например, в атрибуте alt тега img ).
 3 ) Если они находятся внутри заголовков ( теги h1 - h6 ).
 4 ) Если они являются составляющей частью другого слова ( например, термин
 рот, найденное слово - крот ).
 
 Кроме того, вы можете указать где заменять термины, в тексте поста и/или
 комментариях.
*/

class Terms_descriptions {
	/**
	 * Этот метод выполняет обработку запросов со страницы настроек
	 */
	public function management_handler() {
		//загружаем файл переводов ( .mo )
		//они должны называться terms-descriptions-locale.mo
		$plugin_dir = basename( dirname( __FILE__ ) );
		$domain = Terms_descriptions::get_text_domain();
		load_plugin_textdomain( $domain, '/wp-content/plugins/'.$plugin_dir, $plugin_dir );

		//подключаем JS скрипт, выполняющий переключение списков постов и страниц
		$script_src = get_bloginfo( 'siteurl' ).'/wp-content/plugins/terms-descriptions/pagesposts.js';
		wp_enqueue_script( 'td_pagesposts', $script_src, array( 'jquery' ), false, true );

		$parent_file = 'tools.php?page=' . Terms_descriptions::get_plugin_slug();
		
		//если запрос пришел со страницы плагина
		if ( 'terms-descriptions.php' == substr( $parent_file, -22 ) ) {
			//получаем список терминов
			$terms = get_option( 'td_terms' );
			
			//читаем название команды
			$action = $_GET['action'];
			
			switch ( $action ) {
				//добавляем термин
				case 'add':
					check_admin_referer( 'td-add' );
					if ( !isset( $_GET['term'] ) || trim( $_GET['term'] ) == '' || !isset( $_GET['termpageid'] ) ) {
						$message = 4;
						wp_redirect( "$parent_file&m=$message" );
						break;
					}
					$term = $_GET['term'];
					$page_id = $_GET['termpageid'];
					//сохраняем термин
					$key = Terms_descriptions::getNewKey( $terms );
					//если ссылка внешняя
					if ( 'http:' === substr( $page_id, 0, 5 ) ) {
						$terms[$key] = array(
								'term'=>$term,
								'pageid'=>0,
								'url'=>$page_id,
								'title'=>$page_id,
							);
					}
					//если ссылка на внутреннюю страницу
					else {
						$page = get_page( $page_id );
						$terms[$key] = array(
								'term'=>$term,
								'pageid'=>$page_id,
								'url'=>get_page_link( $page->ID ),
								'title'=>$page->post_title,
							);
					}
					update_option( 'td_terms', $terms );
					$message = 1;
					wp_redirect( "$parent_file&m=$message" );
					break;
				//удаляем термин
				case 'delete':
					check_admin_referer( 'delete_term' );
					if ( isset( $_GET['termid'] ) && is_numeric( $_GET['termid'] )
							&& isset( $terms[$_GET['termid']] )) {
						unset( $terms[$_GET['termid']] );
						update_option( 'td_terms', $terms );
						$message = 2;
					}
					else {
						$message = 5;
					}
					wp_redirect( "$parent_file&m=$message" );
					break;
				//изменяем термин
				case 'edit':
					check_admin_referer( 'td-edit' );
					if ( isset( $_GET['termid'] ) && is_numeric( $_GET['termid'] )
							&& isset( $terms[$_GET['termid']] )) {
						$term = $_GET['term'];
						$page_id = $_GET['termpageid'];
						//если ссылка внешняя
						if ( 'http:' === substr( $page_id, 0, 5 ) ) {
							$terms[$_GET['termid']] = array(
								'term'=>$term,
								'pageid'=>0,
								'url'=>$page_id,
								'title'=>$page_id,
							);
						}
						//если ссылка на внутреннюю страницу
						else {
							$page = get_page( $page_id );
							$terms[$_GET['termid']] = array(
								'term'=>$term,
								'pageid'=>( int )$page_id,
								'url'=>get_page_link( $page->ID ),
								'title'=>$page->post_title,
							);
						}
						update_option( 'td_terms', $terms );
						$message = 3;
					}
					else {
						$message = 6;
					}
					wp_redirect( "$parent_file&m=$message" );
					break;
			}
		}
	}
	
	/**
	 * Возвращает $menu_slug плагина ( задается в функции add_management_page )
	 * ( terms_descriptions/terms-descriptions.php )
	 */
	public function get_plugin_slug() {
		return $_GET['page'];
	}
	
	/**
	 * Возвращает уникальный идентификатор текстового домена для локализации
	 */
	public function get_text_domain() {
		return 'terms-descriptions';
	}
	
	/**
	 * Возвращает id нового термина
	 */
	public function getNewKey( $terms ) {
		//определяем максимальный существующий id в массиве и возвращаем id+1,
		//если термины еще не созданы, возвращаем 1
		if ( false == $terms ) {
			return 1;
		}
		else {
			krsort( $terms, SORT_NUMERIC );
			return ( int )key( $terms ) + 1;
		}
	}
	  /**
	 * Этот метод добавляет в админку страницу с настройками плагина
	 */
	public function addPages() {
		$domain = Terms_descriptions::get_text_domain();
		
		add_management_page( __( 'Terms Descriptions', $domain ), __( 'Terms Descriptions', $domain ), "manage_options", __FILE__, array( 'Terms_descriptions', 'manage_terms' ));
	}
	
	/**
	 * Этот метод создаёт страницу с настройками плагина
	 */
	public function manage_terms() {
		$domain = Terms_descriptions::get_text_domain();
		//на эту страницу редиректят все обработчики команд, и при этом они
		//передают свои сообщения, которые нужно вывести
		if ( isset( $_GET['updated'] )) {
			echo '<div id="message" class="updated fade"><p>'.__( 'Options saved', $domain ).'</p></div>';
		}
		else if ( isset( $_GET['m'] ) && is_numeric( $_GET['m'] )) {
			$message = $_GET['m'];
			$messages[1] = __( 'Term created', $domain );
			$messages[2] = __( 'Term deleted', $domain );
			$messages[3] = __( 'Term updated', $domain );
			$messages[4] = __( 'Term create error', $domain );
			$messages[5] = __( 'Term delete error', $domain );
			$messages[6] = __( 'Term update error', $domain );
			echo '<div id="message" class="updated fade"><p>'.$messages[$message].'</p></div>';
		}
		
		$terms = get_option( 'td_terms' );
		   //дальше идет разметка страницы
?>
<div class="wrap">
	<div id="icon-edit" class="icon32"><br/></div>
	<h2><?php _e( 'Terms descriptions plugin', $domain ); ?></h2>
	<br class="clear" />
	 <div id="col-container">
		<div id="col-right">
			<div class="col-wrap">
			<?php Terms_descriptions::show_terms( $terms ); ?>
			</div>
		</div>
		<div id="col-left">
			<div class="col-wrap">
				<?php
				//выводим форму создания или изменения термина
				if ( 'term_edit' == $_GET['action'] && !empty( $_GET['termid'] )
						&& is_numeric( $_GET['termid'] )) {
					Terms_descriptions::show_term_form( $_GET['termid'], $terms[$_GET['termid']] );
				}
				else {
					Terms_descriptions::show_term_form();
				}
				?>
			</div>
		</div>
	 </div>
</div>
<?php
	}
	
	/**
	 * Создаёт разметку форму добавления нового термина или редактирования
	 * существующего и форму с настройками
	 */
	public function show_term_form( $term_id = '', $term = null ) {
		$domain = Terms_descriptions::get_text_domain();
		
		//если нужно показать форму изменения термина
		if ( !empty( $term_id ) && isset( $term )) {
			$form_title = __( 'Update term', $domain );
			$submit_text = __( 'Update', $domain );
			$form_open_tag = '<form name="editterm" id="editterm" method="get" action="" class="validate">';
			$action = 'edit';
			$nonse = 'td-edit';
			$term_id_field = '<input type="hidden" id="termid" name="termid" value="'.$term_id.'" />';
			//проверяем указывает ли ссылка на внешнюю страницу или на внутреннюю
			if ( 0 === ( int )$term['pageid'] ) {
				$page_type = 'external';
				$page_url = $term['url'];
			}
			else {
				$page_type = get_post_type( ( int )$term['pageid'] );
				$page_url = 'http://';
			}
		}
		else {
			$form_title = __( 'Create term', $domain );
			$submit_text = __( 'Create', $domain );
			$form_open_tag = '<form name="addterm" id="addterm" method="get" action="" class="validate">';
			$action = 'add';
			$nonse = 'td-add';
			$term_id_field = '';
			$page_url = 'http://';
		}
?>
<div class="form-wrap">
	<h3><?php echo $form_title; ?></h3>
	<div id="ajax-response"></div>
	<?php echo $form_open_tag; ?>
		<input type="hidden" name="page" value="<?php echo Terms_descriptions::get_plugin_slug() ?>"/>
		<input type="hidden" name="action" value="<?php echo $action ?>" />
		<?php echo $term_id_field; ?>
		<?php wp_original_referer_field( true, 'previous' ); wp_nonce_field( $nonse ); ?>
		<div class="form-field form-required">
			<label for="term"><?php _e( 'Term', $domain ); ?></label>
			<input name="term" id="term" type="text" value="<?php echo ( isset( $term )) ? $term['term'] : ''; ?>" size="20" />
			<p><?php _e( 'Term can contain one or several words and will be converted to a link', $domain ); ?>.</p>
		</div>
		<div class="form-field form-required">
			<label for="termtarget"><?php _e( 'Link to', $domain ); ?></label>
			<select name="termtarget" id="termtarget">
				<?php $selected = ( 'page' == $page_type ) ? 'selected="selected"' : ''; ?>
				<option value="pages" <?php echo $selected; ?>><?php _e( 'Page', $domain ) ?></option>
				<?php $selected = ( 'post' == $page_type ) ? 'selected="selected"' : ''; ?>
				<option value="posts" <?php echo $selected; ?>><?php _e( 'Post', $domain ) ?></option>
				<?php $selected = ( 'external' == $page_type ) ? 'selected="selected"' : ''; ?>
				<option value="external" <?php echo $selected; ?>><?php _e( 'External link', $domain ) ?></option>
			</select>
		</div>
		<div class="form-field form-required">
			<label for="termpageid"><?php _e( 'Select page/post or enter external link', $domain ); ?></label>
			<?php if ( 'external' !== $page_type ) { ?>
				<select name="termpageid" id="termpageid">
					<option>---</option>
				</select>
			<?php } else { ?>
				<input type="text" name="termpageid" id="termpageid" value="<?php echo $page_url; ?>" />
			<?php } ?>
			<p><?php _e( 'Select a page or post you want to link to', $domain ); ?>.</p>
		</div>
		<p class="submit">
			<?php if ( 'edit' == $action ) echo '<a accesskey="c" title="'.__( 'Cancel', $domain ).'" class="cancel button-secondary alignright" href="tools.php?page=' . Terms_descriptions::get_plugin_slug() . '">'.__( 'Cancel', $domain ).'</a>' ?>
			<input type="submit" class="button-primary alignleft" name="submit" value="<?php echo $submit_text; ?>" />
		</p>
	</form>
</div>
<div class="form-wrap">
	<h3><?php _e( 'Options', $domain ); ?></h3>
	<form action="options.php" method="post" name="showon" id="showon">
		<?php settings_fields( 'td-settings-group' ); ?>
		<table class="form-table">
			<tbody>
			<tr valign="top">
				<th scope="row"><?php _e( 'Convert terms to links', $domain ); ?></th>
				<td>
				<fieldset>
					<legend class="screen-reader-text"><span><?php _e( 'Convert terms to links', $domain ); ?></span></legend>
					<?php
						$show_on = get_option( 'td_target' );
						$t_posts = $t_comments = '';
						if ( is_array( $show_on )) {
							$t_posts = ( in_array( 'posts', $show_on ) ) ? 'checked="checked"' : '';
							$t_comments = ( in_array( 'comments', $show_on ) ) ? 'checked="checked"' : '';
						}
					?>
					<label><input type="checkbox" value="posts" name="td_target[]" <?php echo $t_posts; ?> /> <?php _e( 'in posts', $domain ); ?></label><br />
					<label><input type="checkbox" value="comments" name="td_target[]" <?php echo $t_comments; ?> /> <?php _e( 'in comments', $domain ); ?></label>
				</fieldset>
				</td>
			</tr>
			<tr>
				<td colspan="2">
				<fieldset>
					<legend class="screen-reader-text"><span><?php _e( 'Convert first', $domain ); ?></span></legend>
					<?php $convert_count = get_option( 'td_count' ); ?>
					<label><?php _e( 'Convert first', $domain ); ?> <input type="text" value="<?php echo $convert_count; ?>" name="td_count" size="3" /> <?php _e( 'terms', $domain ); ?></label>
					<p><?php _e( 'Enter a number', $domain ); ?>.</p>
				</fieldset>
				</td>
			</tr>
			</tbody>
		</table>
		<p class="submit">
			<input type="submit" class="button-primary alignleft" name="submit" value="<?php _e( 'Save', $domain ); ?>" />
		</p>
	</form>
</div>
<?php
	}
	
	/**
	 * Формирует страницу со списком терминов
	 */
	public function show_terms( $terms ) {
		$domain = Terms_descriptions::get_text_domain();
		
		if ( false == $terms ) {
			echo '<strong>'.__( 'Terms not found', $domain ).'</strong>';
		}
		else {
?>
<table class="widefat">
	<thead>
	<tr>
		<th scope="col"><?php _e( 'Term', $domain ); ?></th>
		<th scope="col"><?php _e( 'Link', $domain ); ?></th>
	</tr>
	</thead>
	<tfoot>
	<tr>
		<th scope="col"><?php _e( 'Term', $domain ); ?></th>
		<th scope="col"><?php _e( 'Link', $domain ); ?></th>
	</tr>
	</tfoot>
	<tbody id="the-list">
	<?php foreach ( $terms as $key => $term ) { ?>
		<tr class="iedit alternate">
			<td class="name column-name">
				<span style="display:block"><strong><?php echo $term['term']; ?></strong></span>
				<div class="row-actions">
					<span class="edit"><a href="tools.php?page=<?php echo Terms_descriptions::get_plugin_slug(); ?>&amp;action=term_edit&amp;termid=<?php echo $key; ?>"><?php _e( 'Edit', $domain ); ?></a></span> |
					<span class="delete">
						<?php
							$link = 'tools.php?page=' . Terms_descriptions::get_plugin_slug() . '&amp;action=delete&amp;termid=' . $key;
							$link = ( function_exists( 'wp_nonce_url' ) ) ? wp_nonce_url( $link, 'delete_term' ) : $link;
						?>
						<a class="submitdelete" href="<?php echo $link ?>" onclick="if ( confirm( '<?php _e( 'Are you sure?', $domain ); ?>' ) ) { return true;}return false;">
							<?php _e( 'Delete', $domain ); ?>
						</a>
					</span>
				</div>
			</td>
			<td class="name column-name">
				<a href="<?php echo $term['url']; ?>"><?php echo $term['title']; ?></a>
			</td>
		</tr>
	<?php } ?>
	</tbody>
</table>
<?php
		}
	}
	
	/**
	 * Регистрирует группу чекбоксов ( в форме настроек )
	 */
	public function registerTdSettings() {
		register_setting( 'td-settings-group', 'td_target' );
		//в третьем параметре указываем имя функции, которая будет вызвана
		//перед сохранением полученного значения ( в данном случае - преобразование в число )
		register_setting( 'td-settings-group', 'td_count', 'intval' );
	}
	
	/**
	 * Заменяет термины на ссылки
	 */
	public function termReplace( $content ) {
		//определяем id данной страницы или поста
		$cur_id = get_the_id();
		//определяем максимальное количество замен
		$replace_count = get_option( 'td_count' );
		if ( false == $replace_count ) {
			$replace_count = -1; //нет ограничений
		}
		$terms = get_option( 'td_terms' );
		if ( $terms ) {
			foreach ( $terms as $term ) {
				//если термин ссылается на данную страницу, пропускаем его
				if ( $cur_id == $term['pageid'] ) {
					continue;
				}
				//нужно пропустить термины, которые находятся внутри ссылок, заголовков ( теги h1..h6 )
				//или внутри атрибутов тегов
				/*
				 Алгоритм:
				 1 ) находим все ссылки и теги
				 2 ) разбиваем текст по ним
				 3 ) заменяем термины в оставшихся фрагментах
				 4 ) учитываем, что перед или термина может стоять скобка, знак препинания и т.п.
				 5 ) если есть ограничение на количество замен, проверяем не превысили ли мы его
				*/
				
				$replace_terms = $replace_count;
				
				preg_match_all( '/<a\s.*?>.*?<\/a>|<h\d.*?\/h\d>|<.*?>/isu', $content,
					$matches, PREG_OFFSET_CAPTURE );
				$result = '';
				$start_pos = 0;
				
				//регулярное выражение для замены терминов
				$replace_re = '/([\s\r\n\:\;\!\?\.\,\)\(<>]{1}|^)('.$term['term'].')([\s\r\n\:\;\!\?\.\,\)\(<>]{1}|$)/isu';
				foreach ( $matches[0] as $match ) {
					//был ли текст до этого совпадания
					$length = $match[1] - $start_pos;
					if ( $length > 0 ) {
						//ищем термин
						$text = substr( $content, $start_pos, $length );
						if ( $replace_terms >= 0 ) {
							$result .= preg_replace( $replace_re, '$1<a href="'.$term['url'].'">$2</a>$3', $text, $replace_terms, $replaced );
							$replace_terms -= $replaced;
						}
						else {
							$result .= preg_replace( $replace_re, '$1<a href="'.$term['url'].'">$2</a>$3', $text );
						}
						
					}
					//добавляем к результату сам тег
					$result .= $match[0];
					$start_pos = $match[1] + strlen( $match[0] );
				}
				//проверяем, обработан ли весь текст
				//( на случай, если $content не содержит закрывающего тега в конце )
				if ( $start_pos < strlen( $content )) {
					$text = substr( $content, $start_pos );
					if ( $replace_terms >= 0 ) {
						$result .= preg_replace( $replace_re, '$1<a href="'.$term['url'].'">$2</a>$3', $text, $replace_terms, $replaced );
						$replace_terms -= $replaced;
					}
					else {
						$result .= preg_replace( $replace_re, '$1<a href="'.$term['url'].'">$2</a>$3', $text );
					}
				}
				$content = $result;
			}
		}
		return $content;
	}
	
	function generate_js() {
		//определяем id поста на который установлена ссылка
		//нужно чтобы установить атрибут selected
		$term_id = ( isset( $_GET['termid'] ) ) ? $_GET['termid'] : null;
		$page_url = 'http://';
		if ( $term_id !== null ) {
			$terms = get_option( 'td_terms' );
			$page_id = $terms[$term_id]['pageid'];
			if ( 0 == $page_id ) {
				$page_url = $terms[$term_id]['url'];
			}
		}
		echo '<script type="text/javascript"> /* <![CDATA[ */'."\n";
		//получаем посты и страницы и формируем списки
		$posts = get_posts();
		if ( count( $posts ) > 0 ) {
			//создаём элементы списка тегов option с перечнем постов
			echo 'var posts = "";'."\n";
			foreach ( $posts as $i => $post ) {
				if ( isset( $page_id ) && $post->ID == $page_id ) {
					$selected = 'selected=\"selected\"';
				}
				else {
					$selected = '';
				}
				echo 'posts += "<option value=\"'.$post->ID.'\" '.$selected.'>'.$post->post_title.'</option>";'."\n";
			}
		}

		$pages = get_pages();
		if ( count( $pages ) > 0 ) {
			//создаём элементы списка тегов option с перечнем страниц
			echo 'var pages = "";'."\n";
			foreach ( $pages as $i => $page ) {
				if ( isset( $page_id ) && $page->ID == $page_id ) {
					$selected = 'selected=\"selected\"';
				}
				else {
					$selected = '';
				}
				echo 'pages += "<option value=\"'.$page->ID.'\" '.$selected.'>'.$page->post_title.'</option>";'."\n";
			}
		}
		
		//сохраняем адрес внешней ссылки (если она установлена)
		echo 'var external_page = "'.$page_url.'";'."\n";
		
		echo '/* ]]> */</script>';
	}
}

//обработчик запросов
add_action( 'admin_init', array( 'Terms_descriptions', 'management_handler' ) );
//обработчик формы с настройками
add_action( 'admin_init', array( 'Terms_descriptions', 'registerTdSettings' ) );

//инсталляция плагина, на данный момент не используется
//register_activation_hook( __FILE__, array( 'Terms_descriptions', 'install' ) );
//деинсталляция плагина, на данный момент не используется
//register_deactivation_hook( __FILE__, array( 'Terms_descriptions', 'uninstall' ) );

//добавляем фильтры для постов и комментариев в которых нужно преобразовывать ссылки в термины
$tdTarget = get_option( 'td_target' );
if ( is_array( $tdTarget ) && in_array( 'posts', $tdTarget )) {
	add_filter( 'the_content', array( 'Terms_descriptions', 'termReplace' ) );
}
if ( is_array( $tdTarget ) && in_array( 'comments', $tdTarget )) {
	add_filter( 'comment_text', array( 'Terms_descriptions', 'termReplace' ) );
}
  
//создание страницы с настройками
add_action( 'admin_menu', array( 'Terms_descriptions', 'addPages' ) );

add_action( 'admin_footer', array( 'Terms_descriptions', 'generate_js' ) );

//end of terms-descriptions.php