<?php  
/* Plugin Name: Top Commentators Widget  
Description: Adds a sidebar widget to show the top commentators in your WP site. Adapted from Show Top Commentators plugin.  
Author: Lorna Timbah (WebGrrrl.net)
Version: 1.3
Author URI: http://webgrrrl.net 
Plugin URI: http://webgrrrl.net/archives/my-top-commentators-widget-quick-dirty.htm
*/    

// Put functions into one big function we'll call at the plugins_loaded  
// action. This ensures that all required plugin functions are defined.

function widget_topcomm_init() {
	// Check for the required plugin functions. This will prevent fatal
  	// errors occurring when you deactivate the dynamic-sidebar plugin.
  	if ( !function_exists('register_sidebar_widget') )
  		return;

	// This is the function that truncates long commentators names to avoid
	// your site design from breaking
	function ns_substr_ellipse($str, $len) {
		if(strlen($str) > $len) {
			$str = substr($str, 0, $len-3) . "...";
		}
		return $str;
	}

	// This is the function that queries whether the commenters have entered a URL
	function ns_get_user_url($user) {
		global $wpdb;
		$url = $wpdb->get_var("
			SELECT comment_author_url
				FROM $wpdb->comments
			WHERE comment_author = '".addslashes($user)."'
			AND comment_author_url != 'http://'
			AND comment_approved = 1
			ORDER BY comment_date DESC LIMIT 1
		");
		return $url;
	}

	// This is the function that writes out the top commentators list
	function ns_show_top_commentators() {
		// Each widget can store its own options. We keep strings here.
		$options = get_option('widget_topcomm');

		// This prepares URLs for filtering
		if($options['filterUrl'] != "") {
			$filterUrl = trim($options['filterUrl']);
			$filterUrl = explode(",", $filterUrl);
			for($i=0; $i<count($filterUrl); $i++) {
				$new_urls .= " AND comment_author_url NOT LIKE '%" . trim($filterUrl[$i]) . "%'";
			}
			$filterUrl = $new_urls;
		}

		// This prepares e-mails for filtering
		if($options['filterEmail'] != "") {
			$filterEmail = trim($options['filterEmail']);
			$filterEmail = explode(",", $filterEmail);
			for($i=0; $i<count($filterUrl); $i++) {
				$new_emails .= " AND comment_author_email NOT LIKE '%" . trim($filterEmail[$i]) . "%'";
			}
			$filterEmail = $new_emails;
		}

		// This prepares names for filtering
		if($options['excludeNames'] != "") {
			$excludeNames = trim($options['excludeNames']);
			$excludeNames = explode(",", $excludeNames);
			for($i=0; $i<count($excludeNames); $i++) {
				$new_names .= " AND comment_author NOT IN ('" . trim($excludeNames[$i]) . "')";
			}
			$excludeNames = $new_names;
		}

		if($options['limitList'] != "")
			$limitList = "LIMIT " . $options['limitList'];
		if($options['limitChar'] != "") {
			$limitChar = $options['limitChar'];
		} else {
			$limitChar = 20;
		}
		$listDesc = $options['listDesc'];
		$listType = $options['listType'];
		$listPeriod = $options['listPeriod'];
		if($options['listNull'] == "") {
			$listNull = "No commentators.";
		} else {
			$listNull = $options['listNull'];
		}
		$makeLink = $options['makeLink'];
		$targetBlank = $options['targetBlank'];
		$noFollow = $options['noFollow'];
		$showCount = $options['showCount'];

		// Gravatar variables by SNascimento
		$displayGravatar = $options['displayGravatar'];
		$avatarSize = $options['avatarSize'];

		// This sets the type of list to be used
		if($listType == "num") {
			$listStart = "<ol>";
			$listEnd = "</ol>";
		} else {
			$listStart = "<ul>";
			$listEnd = "</ul>";
		}

		// This is the function that prepare time period limitation for filtering
		if($listPeriod == "h") {
			$listPeriod = "DATE_FORMAT(comment_date, '%Y-%m-%d %H') = DATE_FORMAT(CURDATE(), '%Y-%m-%d %H')";
		} elseif($listPeriod == "d") {
			$listPeriod = "DATE_FORMAT(comment_date, '%Y-%m-%d') = DATE_FORMAT(CURDATE(), '%Y-%m-%d')";
		} elseif($listPeriod == "w") {
			$listPeriod = "DATE_FORMAT(comment_date, '%Y-%v') = DATE_FORMAT(CURDATE(), '%Y-%v')";
		} elseif($listPeriod == "m") {
			$listPeriod = "DATE_FORMAT(comment_date, '%Y-%m') = DATE_FORMAT(CURDATE(), '%Y-%m')";
		} elseif($listPeriod == "y") {
			$listPeriod = "DATE_FORMAT(comment_date, '%Y') = DATE_FORMAT(CURDATE(), '%Y')";
		} elseif($listPeriod == "a") {
			$listPeriod = "1=1";
		} elseif(is_numeric($listPeriod)) {
			$listPeriod = "comment_date >= CURDATE() - INTERVAL $listPeriod DAY";
		// check if a range of date is entered, then generate the appropriate SQL statement
		} elseif(strpos($listPeriod, 'and') !== false) {
			$listPeriod = "comment_date BETWEEN $listPeriod";
		} else {
			$listPeriod = "DATE_FORMAT(comment_date, '%Y-%m') = DATE_FORMAT(CURDATE(), '%Y-%m')";
		}

		// This writes out the proper SQL whether to group commenters
		// by e-mail or user name
		if($options['groupBy'] == "0") {
			$groupBy = "GROUP BY comment_author";
		} else {
			$groupBy = "GROUP BY comment_author_email";
		}

		// These lines generate our output. Widgets can be very complex
		// but as you can see here, they can also be very, very simple.
		global $wpdb;
		$commenters = $wpdb->get_results("
			SELECT COUNT(comment_author) AS comment_comments, comment_author, comment_author_url, comment_author_email
			FROM $wpdb->comments
			WHERE comment_type != 'pingback'
			AND comment_author != ''
			AND comment_approved = '1'
			AND $listPeriod
			$excludeNames
			$filterUrl
			$filterEmail
			$groupBy
			ORDER BY comment_comments DESC, comment_author ASC
			$limitList
			");

		if(is_array($commenters)) {
			$countList = 0;
			echo $listDesc . "\n";
			echo $listStart . "\n";
			foreach ($commenters as $k) {
				$url = ns_get_user_url($k->comment_author);

// This writes out the list of commentors and checks for 1) Hyperlink each name,
// 2) NoFollow each name, 3) Limit characters in names, 4) Remarks for blank list, and
// 5) Display only users with URLs variables

// check if onlyWithUrl = 1
if($options['onlyWithUrl'] == '1') {
// if onlyWithUrl = 1, check if trimurl != ''
// write
if(trim($url) != '') {
				echo "<li>";
				if(trim($url) != '' ) {
					if($makeLink == 1) {
						echo "<a href='" . $url . "'";
						if($noFollow == 1) echo " rel='nofollow'";
						if($targetBlank == 1) echo " target='_blank'";
						echo ">";
					}
				}

				// Gravatar display by SNascimento
				if($displayGravatar == 1)  {
					$image=md5(strtolower($k->comment_author_email));
					$defavatar=urlencode($defaultGravatar);
					echo '<img class="tcwGravatar" src="http://www.gravatar.com/avatar.php?gravatar_id='.$image.'&amp;size='.$avatarSize.'&amp;default='.$defavatar.'" alt ="'.$k->comment_author.'" title="'.$k->comment_author.'" /> ';
				}

				echo ns_substr_ellipse($k->comment_author, $limitChar);
				if($showCount == 1) echo " (" . $k->comment_comments . ")";
				if(trim($url) != '') {
					if($makeLink == 1)
						echo "</a>";
				}
				echo "</li>\n";
				unset($url);
				$countList = $ $countList + 1;
// finish checking trimurl
}
// if onlyWithUrl = 0
// write
} else {
				echo "<li>";
				if(trim($url) != '' ) {
					if($makeLink == 1) {
						echo "<a href='" . $url . "'";
						if($noFollow == 1) echo " rel='nofollow'";
						if($targetBlank == 1) echo " target='_blank'";
						echo ">";
					}
				}

				// Gravatar display by SNascimento
				if($displayGravatar == 1)  {
					$image=md5(strtolower($k->comment_author_email));
					$defavatar=urlencode($defaultGravatar);
					echo '<img class="tcwGravatar" src="http://www.gravatar.com/avatar.php?gravatar_id='.$image.'&amp;size='.$avatarSize.'&amp;default='.$defavatar.'" alt ="'.$k->comment_author.'" title="'.$k->comment_author.'" border="0"/> ';
				}

				echo ns_substr_ellipse($k->comment_author, $limitChar);
				if($showCount == 1) echo " (" . $k->comment_comments . ")";
				if(trim($url) != '') {
					if($makeLink == 1)
						echo "</a>";
				}
				echo "</li>\n";
				unset($url);
				$countList = $countList + 1;
// end check onlyWithUrl
}
			}
			if($countList == 0)
				echo "<li>" . $listNull . "</li>\n";
			echo $listEnd;
		} else {
			echo "<ul><li>" . $listNull . "</li></ul>" . "\n";
		}
	}

   	// This is the function that outputs our top commentators list
  	function widget_topcomm($args) {
 		// $args is an array of strings that help widgets to conform to
  		// the active theme: before_widget, before_title, after_widget,
  		// and after_title are the array keys. Default tags: li and h2.
  		extract($args);

   		// This one string determines whether you want it to appear in the main page or everywhere
  		$options = get_option('widget_topcomm');
  		$showInHome = $options['showInHome'];
  		$title = htmlspecialchars(stripcslashes($options['title']), ENT_QUOTES);
  		global $wpdb;
  		if($showInHome == 1) {
  			if(is_home())
   				echo $before_widget . $before_title . $title . $after_title;
  		} else {
  			echo $before_widget . $before_title . $title . $after_title;
  		}
		if($showInHome == 1) {
			if(is_home())
				echo ns_show_top_commentators();
		} else {
			echo ns_show_top_commentators();
		}
		if($showInHome == 1) {
			if(is_home())
	  			echo $after_widget . "<!-- end of widget -->";
		} else {
  			echo $after_widget . "<!-- end of widget -->";
		}
	}

	function widget_topcomm_control() {
		// Get our options and see if we're handling a form submission.
		$options = get_option('widget_topcomm');
		if (!is_array($options) )
			$options = array('title'=>'Top Commentators', 'listDesc'=>'You commented; therefore you are loved:', 'excludeNames'=>'\'Lorna\', \'Administrator\'','limitList'=>'10', 'limitChar'=>'25', 'listNull'=>'Be the first to comment in my site.', 'filterUrl'=>'', 'filterEmail'=>'', 'listType'=>'bul', 'listPeriod'=>'m', 'makeLink'=>'1', 'targetBlank'=>'0', 'noFollow'=>'1', 'showCount'=>'1', 'groupBy'=>'1', 'showInHome'=>'0', 'onlyWithUrl'=>'0', 'displayGravatar'=>'0', 'avatarSize'=>'20');
		if ( $_POST['topcomm-submit'] ) {
			// Remember to sanitize and format use input appropriately.
			$options['title'] = $_POST['topcomm-title'];
			$options['excludeNames'] = $_POST['topcomm-excludeNames'];
			$options['limitList'] = $_POST['topcomm-limitList'];
			$options['limitChar'] = $_POST['topcomm-limitChar'];
			$options['listDesc'] = $_POST['topcomm-listDesc'];
			$options['listType'] = $_POST['topcomm-listType'];
			if($_POST['topcomm-listPeriodnum'] == '') {
				$options['listPeriod'] = $_POST['topcomm-listPeriod'];
			} else {
				$options['listPeriod'] = $_POST['topcomm-listPeriodnum'];
			}
			$options['listNull'] = $_POST['topcomm-listNull'];
			$options['filterUrl'] = $_POST['topcomm-filterUrl'];
			$options['filterEmail'] = $_POST['topcomm-filterEmail'];
			$options['makeLink'] = $_POST['topcomm-makeLink'];
			$options['targetBlank'] = $_POST['topcomm-targetBlank'];
  			$options['noFollow'] = $_POST['topcomm-noFollow'];
			$options['showInHome'] = $_POST['topcomm-showInHome'];
			$options['onlyWithUrl'] = $_POST['topcomm-onlyWithUrl'];
			$options['showCount'] = $_POST['topcomm-showCount'];
			$options['groupBy'] = $_POST['topcomm-groupBy'];
			// Gravatar options
			$options['displayGravatar'] = $_POST['topcomm-displayGravatar'];
			$options['avatarSize'] = ($_POST['topcomm-avatarSize'] ? $_POST['topcomm-avatarSize'] : 20) ;
  			update_option('widget_topcomm', $options);
  		}
   		// Be sure you format your options to be valid HTML attributes.
    		// Here is our little form segment. Notice that we don't need a
  		// complete form. This will be embedded into the existing form.
		$options = get_option('widget_topcomm');
		$title = htmlspecialchars(stripcslashes($options['title']), ENT_QUOTES);
		$excludeNames = htmlspecialchars(stripcslashes($options['excludeNames']), ENT_QUOTES);
		$limitList = $options['limitList'];
		$limitChar = $options['limitChar'];
		$listDesc = $options['listDesc'];
		$listType = $options['listType'];
		$listPeriod = $options['listPeriod'];
		$listNull = $options['listNull'];
		$filterUrl = $options['filterUrl'];
		$filterEmail = $options['filterEmail'];
		$makeLink = $options['makeLink'];
		$targetBlank = $options['targetBlank'];
		$noFollow = $options['noFollow'];
		$showInHome = $options['showInHome'];
		$onlyWithUrl = $options['onlyWithUrl'];
		$showCount = $options['showCount'];
		$groupBy = $options['groupBy'];
		$displayGravatar = $options['displayGravatar'];
		$avatarSize = $options['avatarSize'];
		?>
  		<p style="text-align:right;"><label for="topcomm-title">Change widget title:</label> <input style="width: 200px;" id="topcomm-title" name="topcomm-title" type="text" value="<?php echo $title ?>" /></p>
		<p style="text-align:right;"><label for="topcomm-listDesc">Add description below the title:</label> <input style="width: 200px;" id="topcomm-listDesc" name="topcomm-listDesc" type="text" value="<?php echo $listDesc ?>" /><br />Leave blank to exclude description.</p>
		<p style="text-align:right;"><label for="topcomm-excludeNames">Exclude these users:</label> <input style="width: 200px;" id="topcomm-excludeNames" name="topcomm-excludeNames" type="text" value="<?php echo $excludeNames ?>" /><br />Separate each name with a comma (,)</p>

		<p style="text-align:right;"><label for="topcomm-listPeriod">Reset list every:</label> <select size="1" id="topcomm-listPeriod" name="topcomm-listPeriod"><option value="h" <?php
  		if($listPeriod== 'h')
  			echo ' selected';
  		echo '>Hour</option><option value="d"';
  		if($listPeriod== 'd')
  			echo ' selected';
  		echo '>Day</option><option value="w"';
  		if($listPeriod== 'w')
  			echo ' selected';
  		echo '>Week</option><option value="m"';
  		if($listPeriod== 'm' || is_null($listPeriod))
  			echo ' selected';
  		echo '>Month</option><option value="y"';
  		if($listPeriod== 'y')
  			echo ' selected';
  		echo '>Year</option><option value="a"';
  		if($listPeriod== 'a')
  			echo ' selected';
  		echo '>List all</option></select><br />Or specify number of days / enter range of date: <input style="width: 200px;" id="topcomm-listPeriodnum" name="topcomm-listPeriodnum" type="text" value="';
  		if (is_numeric($listPeriod) || (strpos($listPeriod, 'and') !== false))
   		 	echo $listPeriod;
  		echo '" /><br />E.g. <strong>100</strong> for # of days or <strong>20090301 and 20090531</strong> for date range</p>';
		?>
  		<p style="text-align:right;"><label for="topcomm-limitList">Limit number of names to:</label>  <input style="width: 200px;" id="topcomm-limitList" name="topcomm-limitList" type="text" value="<?php echo $limitList ?>" /><br />Enter numbers only</p>
  		<p style="text-align:right;"><label for="topcomm-limitChar">Limit characters in names to:</label> <input style="width: 200px;" id="topcomm-limitChar" name="topcomm-limitChar" type="text" value="<?php echo $limitChar ?>" /><br />Enter numbers only</p>
  		<p style="text-align:right;"><label for="topcomm-listNull">Remarks for blank list:</label> <input style="width: 200px;" id="topcomm-listNull" name="topcomm-listNull" type="text" value="<?php echo $listNull ?>" /></p>
  		<p style="text-align:right;"><label for="topcomm-filterUrl">Filter the following full/partial URLs:</label> <input style="width: 200px;" id="topcomm-filterUrl" name="topcomm-filterUrl" type="text" value="<?php echo $filterUrl ?>" /><br />Separate each URl with a comma (,)</p>
  		<p style="text-align:right;"><label for="topcomm-filterEmail">Filter the following full/partial e-mail:</label> <input style="width: 200px;" id="topcomm-filterEmail" name="topcomm-filterEmail" type="text" value="<?php echo $filterEmail ?>" /><br />Separate each e-mail with a comma (,)</p>
		<?php
  		echo '<p style="text-align:right;"><label for="topcomm-listType">' . ('Display list type as:') . '</label><select size="1" id="topcomm-listType" name="topcomm-listType"><option value="bul"';
  		if($listType == 'bul')
  			echo ' selected';
  		echo '>Bulleted</option><option value="num"';
  		if($listType == 'num')
  			echo ' selected';
  		echo '>Numbered</option></select></p>';
  		echo '<p style="text-align:right;"><label for="topcomm-makeLink">' . ('Hyperlink each name?') . '</label><select size="1" id="topcomm-makeLink" name="topcomm-makeLink"><option value="1"';
  		if($makeLink == 1)
  			echo ' selected';
 		echo '>Yes</option><option value="0"';
  		if($makeLink == 0)
  			echo ' selected';
  		echo '>No</option></select></p>';

  		echo '<p style="text-align:right;"><label for="topcomm-targetBlank">' . ('Open each link in a new window?') . '</label><select size="1" id="topcomm-targetBlank" name="topcomm-targetBlank"><option value="1"';
  		if($targetBlank == 1)
  			echo ' selected';
 		echo '>Yes</option><option value="0"';
  		if($targetBlank == 0)
  			echo ' selected';
  		echo '>No</option></select></p>';

  		echo '<p style="text-align:right;"><label for="topcomm-noFollow">' . ('NoFollow each name if hyperlinked?') . '</label><select size="1" id="topcomm-noFollow" name="topcomm-noFollow"><option value="1"';
  		if($noFollow == 1)
  			echo ' selected';
 		echo '>Yes</option><option value="0"';
  		if($noFollow== 0)
  			echo ' selected';
  		echo '>No</option></select></p>';

  		echo '<p style="text-align:right;"><label for="topcomm-showCount">' . ('Show number of comments for each commenter?') . '</label><select size="1" id="topcomm-showCount" name="topcomm-showCount"><option value="1"';
  		if($showCount == 1)
  			echo ' selected';
 		echo '>Yes</option><option value="0"';
  		if($showCount == 0)
  			echo ' selected';
  		echo '>No</option></select></p>';

  		echo '<p style="text-align:right;"><label for="topcomm-groupBy">' . ('(Hijack-proof?) Group commentors based on') . '</label><select size="1" id="topcomm-groupBy" name="topcomm-groupBy"><option value="1"';
  		if($groupBy == 1)
  			echo ' selected';
 		echo '>E-mail</option><option value="0"';
  		if($groupBy == 0)
  			echo ' selected';
  		echo '>User names</option></select></p>';

  		echo '<p style="text-align:right;"><label for="topcomm-showInHome">' . ('Show in home page only?') . '</label><select size="1" id="topcomm-showInHome" name="topcomm-showInHome"><option value="1"';
  		if($showInHome == 1)
  			echo ' selected';
 		echo '>Yes</option><option value="0"';
  		if($showInHome == 0)
  			echo ' selected';
  		echo '>No</option></select></p>';

  		echo '<p style="text-align:right;"><label for="topcomm-onlyWithUrl">' . ('Display only commentors with URL?') . '</label><select size="1" id="topcomm-onlyWithUrl" name="topcomm-onlyWithUrl"><option value="1"';
  		if($onlyWithUrl == 1)
  			echo ' selected';
 		echo '>Yes</option><option value="0"';
  		if($onlyWithUrl == 0)
  			echo ' selected';
  		echo '>No</option></select></p>';

		// Gravatar option forms
  		echo '<p style="text-align:right;"><label for="topcomm-displayGravatar">' . ('Display Gravatar?') . '</label><select size="1" id="topcomm-displayGravatar" name="topcomm-displayGravatar"><option value="1"';
  		if($displayGravatar == 1)
  			echo ' selected';
 		echo '>Yes</option><option value="0"';
  		if($displayGravatar == 0)
  			echo ' selected';
  		echo '>No</option></select></p>';

		?>

		<p style="text-align:right;"><label for="topcomm-avatarSize">Gravatar Size:</label> <input style="width: 200px;" id="topcomm-avatarSize" name="topcomm-avatarSize" type="text" value="<?php echo $avatarSize ?>" /></p>

		<?php

  		echo '<input type="hidden" id="topcomm-submit" name="topcomm-submit" value="1" />';
  	}
	// This registers our widget so it appears with the other available
  	// widgets and can be dragged and dropped into any active sidebars.
  	register_sidebar_widget(array('Top Commentators', 'widgets'), 'widget_topcomm');

    	// This registers our optional widget control form. Because of this
  	// our widget will have a button that reveals a 410x500 pixel form.
  	register_widget_control(array('Top Commentators', 'widgets'), 'widget_topcomm_control', 410, 500);
}

// Run our code later in case this loads prior to any required plugins.
add_action('widgets_init', 'widget_topcomm_init');
?>